@@ -194,12 +194,20 @@
         } else {
             k = PyObjC_IdToPython(key);
             if (k == NULL) {
-                    PyObjC_GIL_FORWARD_EXC();
+                PyObjC_GIL_FORWARD_EXC();
             }
         }
 
         if (likely(PyDict_CheckExact(value))) {
+#if PY_MAJOR_VERSION == 3
+            v = PyDict_GetItemWithError(value, k);
+            if (v == NULL && PyErr_Occurred()) {
+                PyObjC_GIL_FORWARD_EXC();
+            }
+#else
+
             v = PyDict_GetItem(value, k);
+#endif
             Py_XINCREF(v);
 
         } else {
