@@ -7,7 +7,7 @@
 typedef struct {
     PyObject_HEAD
 
-    ffi_cif*  cif;
+    ffi_cif* cif;
     PyObjCMethodSignature* methinfo;
     void* function;
     PyObject* doc;
@@ -138,6 +138,38 @@ func_call(PyObject* s, PyObject* args, PyObject* kwds)
 
     PyObject* retval;
 
+    if (PyObjC_DeprecationVersion && self->methinfo->deprecated && self->methinfo->deprecated <= PyObjC_DeprecationVersion) {
+        char buf[128];
+
+#if PY_MAJOR_VERSION == 2
+        if (PyString_Check(self->name)) {
+            snprintf(buf, 128, "%s() is a deprecated API (macOS %d.%d)", PyString_AsString(self->name),
+                    self->methinfo->deprecated / 100, self->methinfo->deprecated % 100);
+        } else
+#endif
+        if (PyUnicode_Check(self->name)) {
+#if PY_VERSION_HEX >= 0x03030000
+            snprintf(buf, 128, "%s() is a deprecated API (macOS %d.%d)",
+                    PyUnicode_AsUTF8(self->name),
+                    self->methinfo->deprecated / 100, self->methinfo->deprecated % 100);
+#else
+            PyObject* temp = PyUnicode_AsUTF8String(self->name);
+            if (temp == NULL) {
+                return NULL;
+            }
+            snprintf(buf, 128, "%s() is a deprecated API (macOS %d.%d)", PyString_AsString(temp),
+                    self->methinfo->deprecated / 100, self->methinfo->deprecated % 100);
+            Py_DECREF(temp);
+#endif
+        } else {
+            snprintf(buf, 128, "function is a deprecated API");
+        }
+
+        if (PyErr_Warn(PyObjCExc_DeprecationWarning, buf) < 0) {
+            return NULL;
+        }
+    }
+
     if (self->methinfo->suggestion != NULL) {
         PyErr_SetObject(PyExc_TypeError, self->methinfo->suggestion);
         return NULL;
