@@ -596,7 +596,14 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* meta, BOOL is_native)
 
     PyObjC_Assert(descr == NULL || descr->allowNULL, -1);
     if (meta) {
+#if PY_MAJOR_VERSION == 3
+        d = PyDict_GetItemStringWithError(meta, "null_accepted");
+        if (d == NULL && PyErr_Occurred()) {
+            return -1;
+        }
+#else
         d = PyDict_GetItemString(meta, "null_accepted");
+#endif
         if (d == NULL || PyObject_IsTrue(d)) {
             /* pass */
         } else {
@@ -607,7 +614,14 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* meta, BOOL is_native)
 
 
     if (meta) {
+#if PY_MAJOR_VERSION == 3
+        d = PyDict_GetItemStringWithError(meta, "already_retained");
+        if (d == NULL && PyErr_Occurred()) {
+            return -1;
+        }
+#else
         d = PyDict_GetItemString(meta, "already_retained");
+#endif
         if (d && PyObject_IsTrue(d)) {
             if (descr == NULL || (descr->tmpl && !descr->alreadyRetained)) return -2;
             descr->alreadyRetained = YES;
@@ -619,7 +633,14 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* meta, BOOL is_native)
 
     PyObjC_Assert(descr == NULL || !descr->alreadyCFRetained, -1);
     if (meta) {
+#if PY_MAJOR_VERSION == 3
+        d = PyDict_GetItemStringWithError(meta, "already_cfretained");
+        if (d == NULL && PyErr_Occurred()) {
+            return -1;
+        }
+#else
         d = PyDict_GetItemString(meta, "already_cfretained");
+#endif
         if (d && PyObject_IsTrue(d)) {
             if (descr == NULL || descr->tmpl) return -2;
             descr->alreadyCFRetained = YES;
@@ -630,7 +651,14 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* meta, BOOL is_native)
 
     PyObjC_Assert(descr == NULL || !descr->callableRetained, -1);
     if (meta) {
+#if PY_MAJOR_VERSION == 3
+        d = PyDict_GetItemStringWithError(meta, "callable_retained");
+        if (d == NULL && PyErr_Occurred()) {
+            return -1;
+        }
+#else
         d = PyDict_GetItemString(meta, "callable_retained");
+#endif
         if (d == NULL || PyObject_IsTrue(d)) {
             if (descr == NULL || descr->tmpl) return -2;
             descr->callableRetained = YES;
@@ -641,7 +669,14 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* meta, BOOL is_native)
 
     PyObjC_Assert(descr == NULL || descr->sel_type == NULL, -1);
     if (meta) {
+#if PY_MAJOR_VERSION == 3
+        d = PyDict_GetItemStringWithError(meta, "sel_of_type");
+        if (d == NULL && PyErr_Occurred()) {
+            return -1;
+        }
+#else
         d = PyDict_GetItemString(meta, "sel_of_type");
+#endif
         if (d) {
             if (PyUnicode_Check(d)) {
                 PyObject* bytes = PyUnicode_AsEncodedString(d, NULL, NULL);
@@ -672,7 +707,14 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* meta, BOOL is_native)
     }
 
     if (meta) {
+#if PY_MAJOR_VERSION == 3
+        d = PyDict_GetItemStringWithError(meta, "callable");
+        if (d == NULL && PyErr_Occurred()) {
+            return -1;
+        }
+#else
         d = PyDict_GetItemString(meta, "callable");
+#endif
         if (d) {
             if (descr == NULL || descr->tmpl) return -2;
 
@@ -680,7 +722,14 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* meta, BOOL is_native)
              * the metadata.
              */
             char buffer[64];
+#if PY_MAJOR_VERSION == 3
+            PyObject* a = PyDict_GetItemStringWithError(d, "arguments");
+            if (a == NULL && PyErr_Occurred()) {
+                return -1;
+            }
+#else
             PyObject* a = PyDict_GetItemString(d, "arguments");
+#endif
 
             if (a != NULL) {
                 Py_ssize_t i, len = PyDict_Size(a);
@@ -708,7 +757,14 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* meta, BOOL is_native)
 
     PyObjC_Assert(descr == NULL || !descr->arraySizeInRetval, -1);
     if (meta) {
+#if PY_MAJOR_VERSION == 3
+        d = PyDict_GetItemStringWithError(meta, "c_array_length_in_result");
+        if (d == NULL && PyErr_Occurred()) {
+            return -1;
+        }
+#else
         d = PyDict_GetItemString(meta, "c_array_length_in_result");
+#endif
         if (d != NULL && PyObject_IsTrue(d)) {
             if (descr == NULL || descr->tmpl) return -2;
 
@@ -718,7 +774,14 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* meta, BOOL is_native)
 
     PyObjC_Assert(descr == NULL || !descr->printfFormat, -1);
     if (meta) {
+#if PY_MAJOR_VERSION == 3
+        d = PyDict_GetItemStringWithError(meta, "printf_format");
+        if (d == NULL && PyErr_Occurred()) {
+            return -1;
+        }
+#else
         d = PyDict_GetItemString(meta, "printf_format");
+#endif
         if (d != NULL && PyObject_IsTrue(d)) {
             if (descr == NULL || descr->tmpl) return -2;
 
@@ -727,7 +790,14 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* meta, BOOL is_native)
     }
 
     if (meta) {
+#if PY_MAJOR_VERSION == 3
+        d = PyDict_GetItemStringWithError(meta, "c_array_delimited_by_null");
+        if (d == NULL && PyErr_Occurred()) {
+            return -1;
+        }
+#else
         d = PyDict_GetItemString(meta, "c_array_delimited_by_null");
+#endif
         if (d != NULL && PyObject_IsTrue(d)) {
             if (descr == NULL || descr->tmpl) return -2;
 
@@ -736,7 +806,14 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* meta, BOOL is_native)
     }
 
     if (meta) {
+#if PY_MAJOR_VERSION == 3
+        d = PyDict_GetItemStringWithError(meta, "c_array_of_fixed_length");
+        if (d == NULL && PyErr_Occurred()) {
+            return -1;
+        }
+#else
         d = PyDict_GetItemString(meta, "c_array_of_fixed_length");
+#endif
         if (d != NULL) {
             if (PyLong_Check(d)) {
                 if (descr == NULL || descr->tmpl) return -2;
@@ -761,7 +838,14 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* meta, BOOL is_native)
     }
 
     if (meta) {
+#if PY_MAJOR_VERSION == 3
+        d = PyDict_GetItemStringWithError(meta, "c_array_of_variable_length");
+        if (d == NULL && PyErr_Occurred()) {
+            return -1;
+        }
+#else
         d = PyDict_GetItemString(meta, "c_array_of_variable_length");
+#endif
         if (d != NULL && PyObject_IsTrue(d)) {
             if (descr == NULL || descr->tmpl) return -2;
 
@@ -772,7 +856,14 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* meta, BOOL is_native)
     }
 
     if (meta) {
+#if PY_MAJOR_VERSION == 3
+        d = PyDict_GetItemStringWithError(meta, "c_array_length_in_arg");
+        if (d == NULL && PyErr_Occurred()) {
+            return -1;
+        }
+#else
         d = PyDict_GetItemString(meta, "c_array_length_in_arg");
+#endif
         if (d != NULL) {
             if (PyLong_Check(d)) {
                 if (descr == NULL || descr->tmpl) return -2;
@@ -844,7 +935,14 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* meta, BOOL is_native)
     }
 
     if (meta) {
+#if PY_MAJOR_VERSION == 3
+        d = PyDict_GetItemStringWithError(meta, "type_modifier");
+        if (d == NULL && PyErr_Occurred()) {
+            return -1;
+        }
+#else
         d = PyDict_GetItemString(meta, "type_modifier");
+#endif
         if (d != NULL) {
             if (PyUnicode_Check(d)) {
                 PyObject* bytes = PyUnicode_AsEncodedString(d, NULL, NULL);
@@ -875,7 +973,14 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* meta, BOOL is_native)
     }
 
     if (meta) {
+#if PY_MAJOR_VERSION == 3
+        d = PyDict_GetItemStringWithError(meta, "type");
+        if (d == NULL && PyErr_Occurred()) {
+            return -1;
+        }
+#else
         d = PyDict_GetItemString(meta, "type");
+#endif
 
     } else {
         d = NULL;
@@ -1009,7 +1114,14 @@ process_metadata_dict(PyObjCMethodSignature* methinfo, PyObject* metadata, BOOL
     }
 
     if (metadata) {
+#if PY_MAJOR_VERSION == 3
+        PyObject* retval = PyDict_GetItemStringWithError(metadata, "retval");
+        if (retval == NULL && PyErr_Occurred()) {
+            return -1;
+        }
+#else
         PyObject* retval = PyDict_GetItemString(metadata, "retval");
+#endif
 
         if (retval != NULL) {
             int r = setup_descr(methinfo->rettype, retval, is_native);
@@ -1032,7 +1144,14 @@ process_metadata_dict(PyObjCMethodSignature* methinfo, PyObject* metadata, BOOL
             }
 
             if (retval != NULL) {
+#if PY_MAJOR_VERSION == 3
+                PyObject* av = PyDict_GetItemStringWithError(metadata, "free_result");
+                if (av == NULL && PyErr_Occurred()) {
+                    return -1;
+                }
+#else
                 PyObject* av = PyDict_GetItemString(metadata, "free_result");
+#endif
                 if (av && PyObject_IsTrue(av)) {
                     methinfo->free_result = YES;
                 }
@@ -1042,7 +1161,14 @@ process_metadata_dict(PyObjCMethodSignature* methinfo, PyObject* metadata, BOOL
     }
 
     if (metadata) {
+#if PY_MAJOR_VERSION == 3
+        PyObject* args = PyDict_GetItemStringWithError(metadata, "arguments");
+        if (args == NULL && PyErr_Occurred()) {
+            return -1;
+        }
+#else
         PyObject* args = PyDict_GetItemString(metadata, "arguments");
+#endif
         if (args != NULL && !PyDict_Check(args)) {
             args = NULL;
         }
@@ -1055,7 +1181,15 @@ process_metadata_dict(PyObjCMethodSignature* methinfo, PyObject* metadata, BOOL
                 int r;
 
                 if (args) {
+#if PY_MAJOR_VERSION == 3
+                    d = PyDict_GetItemWithError(args, k);
+                    if (d == NULL && PyErr_Occurred()) {
+                        Py_DECREF(k);
+                        return -1;
+                    }
+#else
                     d = PyDict_GetItem(args, k);
+#endif
                     Py_DECREF(k);
 
                 } else {
@@ -1088,13 +1222,27 @@ process_metadata_dict(PyObjCMethodSignature* methinfo, PyObject* metadata, BOOL
             }
         }
 
+#if PY_MAJOR_VERSION == 3
+        v = PyDict_GetItemStringWithError(metadata, "suggestion");
+        if (v == NULL && PyErr_Occurred()) {
+            return -1;
+        }
+#else
         v = PyDict_GetItemString(metadata, "suggestion");
+#endif
         if (v) {
             methinfo->suggestion = v;
             Py_INCREF(v);
         }
 
+#if PY_MAJOR_VERSION == 3
+        v = PyDict_GetItemStringWithError(metadata, "deprecated");
+        if (v == NULL && PyErr_Occurred()) {
+            return -1;
+        }
+#else
         v = PyDict_GetItemString(metadata, "deprecated");
+#endif
         if (v) {
             if (PyLong_Check(v)) {
                 methinfo->deprecated = (int)PyLong_AsLong(v);
@@ -1110,13 +1258,27 @@ process_metadata_dict(PyObjCMethodSignature* methinfo, PyObject* metadata, BOOL
         }
 
         methinfo->null_terminated_array = NO;
+#if PY_MAJOR_VERSION == 3
+        v = PyDict_GetItemStringWithError(metadata, "c_array_delimited_by_null");
+        if (v == NULL && PyErr_Occurred()) {
+            return -1;
+        }
+#else
         v = PyDict_GetItemString(metadata, "c_array_delimited_by_null");
+#endif
         if (v && PyObject_IsTrue(v)) {
             methinfo->null_terminated_array = YES;
         }
 
         methinfo->arrayArg = -1;
+#if PY_MAJOR_VERSION == 3
+        v = PyDict_GetItemStringWithError(metadata, "c_array_length_in_arg");
+        if (v == NULL && PyErr_Occurred()) {
+            return -1;
+        }
+#else
         v = PyDict_GetItemString(metadata, "c_array_length_in_arg");
+#endif
         if (v) {
             if (PyLong_Check(v)) {
                 methinfo->arrayArg = (int)PyLong_AsLong(v);
@@ -1132,7 +1294,15 @@ process_metadata_dict(PyObjCMethodSignature* methinfo, PyObject* metadata, BOOL
         }
 
         methinfo->variadic = NO;
+
+#if PY_MAJOR_VERSION == 3
+        v = PyDict_GetItemStringWithError(metadata, "variadic");
+        if (v == NULL && PyErr_Occurred()) {
+            return -1;
+        }
+#else
         v = PyDict_GetItemString(metadata, "variadic");
+#endif
         if (v && PyObject_IsTrue(v)) {
             methinfo->variadic = YES;
 
@@ -1179,7 +1349,14 @@ compiled_metadata(PyObject* metadata)
     PyObjC_Assert(PyDict_Check(metadata), NULL);
 
 
+#if PY_MAJOR_VERSION == 3
+    PyObject* arguments = PyDict_GetItemStringWithError(metadata, "arguments");
+    if (arguments == NULL && PyErr_Occurred()) {
+        return NULL;
+    }
+#else
     PyObject* arguments = PyDict_GetItemString(metadata, "arguments");
+#endif
     if (arguments == NULL || !PyDict_Check(arguments)) {
         max_idx = 0;
     } else {
