@@ -82,6 +82,7 @@ PyObjCClass_HiddenSelector(PyObject* tp, SEL sel, BOOL classMethod)
                     PyErr_Clear();
 
                 } else {
+                    /* XXX: Should change to PyDict_GetItemWithError */
                     PyObject* r = PyDict_GetItem(hidden, v);
                     Py_DECREF(v);
                     if (r == NULL) {
@@ -426,7 +427,18 @@ class_init(PyObject *cls, PyObject *args, PyObject *kwds)
 {
     if (kwds != NULL) {
         if (PyDict_Check(kwds) && PyDict_Size(kwds) == 1) {
-            if (PyDict_GetItemString(kwds, "protocols") != NULL) {
+
+#if PY_MAJOR_VERSION == 3
+            PyObject* v = PyDict_GetItemStringWithError(kwds, "protocols");
+            if (v == NULL && PyErr_Occurred()) {
+                return -1;
+            }
+#else
+            PyObject* v = PyDict_GetItemString(kwds, "protocols");
+#endif
+
+            /* XXX: Not clear what this tries to accomplish */
+            if (v != NULL) {
                 return PyType_Type.tp_init(cls, args, NULL);
             }
         }
@@ -438,7 +450,8 @@ static PyObject*
 class_new(PyTypeObject* type __attribute__((__unused__)),
         PyObject* args, PyObject* kwds)
 {
-static    char* keywords[] = { "name", "bases", "dict", "protocols", NULL };
+static PyObject* all_python_classes = NULL;
+static char* keywords[] = { "name", "bases", "dict", "protocols", NULL };
     char* name;
     PyObject* bases;
     PyObject* dict;
@@ -451,7 +464,7 @@ static    char* keywords[] = { "name", "bases", "dict", "protocols", NULL };
     Py_ssize_t len;
     Class objc_class = NULL;
     Class super_class = NULL;
-    PyObject*  py_super_class = NULL;
+    PyObject* py_super_class = NULL;
     PyObjCClassObject* info;
     PyObject* keys;
     PyObject* protocols;
@@ -909,7 +922,7 @@ static    char* keywords[] = { "name", "bases", "dict", "protocols", NULL };
         return NULL;
     }
 
-    if (PyObjC_MakeBundleForClass != NULL) {
+    if (PyObjC_MakeBundleForClass != NULL && PyObjC_MakeBundleForClass != Py_None) {
         PyObject* m = PyObject_CallObject(PyObjC_MakeBundleForClass, NULL);
         if (m == NULL) {
             (void)PyObjCClass_UnbuildClass(objc_class);
@@ -921,6 +934,10 @@ static    char* keywords[] = { "name", "bases", "dict", "protocols", NULL };
             Py_DECREF(hiddenClassSelectors);
             return NULL;
         }
+        if (PyObjCPythonSelector_Check(m)) {
+            ((PyObjCSelector*)m)->sel_class = objc_class;
+        }
+
         r = PyDict_SetItemString(dict, "bundleForClass", m);
         Py_DECREF(m);
         if (r == -1) {
@@ -971,7 +988,7 @@ static    char* keywords[] = { "name", "bases", "dict", "protocols", NULL };
      * of magic intermediate classes, therefore explicitly refer to the
      * metatype we just created.
      */
-    res =  PyType_Type.tp_new(metatype, args, NULL);
+    res = PyType_Type.tp_new(metatype, args, NULL);
     Py_DECREF(metatype);
     if (res == NULL) {
         Py_DECREF(args);
@@ -1068,6 +1085,16 @@ static    char* keywords[] = { "name", "bases", "dict", "protocols", NULL };
     }
 
     /* This is an "extra" ref */
+    if (all_python_classes == NULL) {
+        all_python_classes = PyList_New(0);
+        if (all_python_classes == NULL) {
+            Py_FatalError("Cannot create internal list");
+        }
+    }
+    if (PyList_Append(all_python_classes, res) < 0) {
+        Py_FatalError("Cannot store generated class");
+    }
+
     Py_INCREF(res);
     return res;
 }
@@ -1100,8 +1127,9 @@ class_dealloc(PyObject* cls)
 
     snprintf(buf, sizeof(buf), "Deallocating objective-C class %s", ((PyTypeObject*)cls)->tp_name);
 
-    /* This should never happen */
-    Py_FatalError(buf);
+    fputs(buf, stderr);
+    Py_INCREF(cls);
+    return;
 }
 
 int
@@ -1119,7 +1147,7 @@ PyObjCClass_CheckMethodList(PyObject* cls, int recursive)
             int r;
             info->generation = PyObjC_MappingCount;
 
-            r =  update_convenience_methods(cls);
+            r = update_convenience_methods(cls);
             if (r < 0) {
                 return -1;
             }
@@ -1143,10 +1171,10 @@ static PyObject*
 metaclass_dir(PyObject* self)
 {
     PyObject* result;
-    Class     cls;
-    Method*   methods;
+    Class cls;
+    Method* methods;
     unsigned int method_count, i;
-    char      selbuf[2048];
+    char selbuf[2048];
 
     /* Start of with keys in __dict__ */
     result = PyDict_Keys(((PyTypeObject*)self)->tp_dict);
@@ -1211,9 +1239,9 @@ _type_lookup(PyTypeObject* tp, PyObject* name
     PyObject *descr = NULL;
     PyObject* res;
 #ifndef PyObjC_FAST_UNICODE_ASCII
-    SEL      sel = PyObjCSelector_DefaultSelector(PyBytes_AsString(name_bytes));
+    SEL sel = PyObjCSelector_DefaultSelector(PyBytes_AsString(name_bytes));
 #else
-    SEL      sel = PyObjCSelector_DefaultSelector(PyObjC_Unicode_Fast_Bytes(name));
+    SEL sel = PyObjCSelector_DefaultSelector(PyObjC_Unicode_Fast_Bytes(name));
 #endif
 
     /* TODO: if sel.startswith('__') and sel.endswith('__'): look_in_runtime = False */
@@ -1256,6 +1284,8 @@ _type_lookup(PyTypeObject* tp, PyObject* name
             descr = PyObjCMetaClass_TryResolveSelector(base, name, sel);
             if (descr != NULL) {
                 break;
+            } else if (PyErr_Occurred()) {
+                return NULL;
             }
         }
     }
@@ -1286,10 +1316,10 @@ _type_lookup_harder(PyTypeObject* tp, PyObject* name
     n = PyTuple_GET_SIZE(mro);
     for (i = 0; i < n; i++) {
         Class cls;
-        Method*   methods;
+        Method*  methods;
         unsigned int method_count, j;
-        char      selbuf[2048];
-        char*     sel_name;
+        char selbuf[2048];
+        char*  sel_name;
 
         base = PyTuple_GET_ITEM(mro, i);
 
@@ -1416,9 +1446,9 @@ _type_lookup_instance(PyObject* class_dict, PyTypeObject* tp, PyObject* name
     PyObject *descr = NULL;
     PyObject* res;
 #ifndef PyObjC_FAST_UNICODE_ASCII
-    SEL      sel = PyObjCSelector_DefaultSelector(PyBytes_AsString(name_bytes));
+    SEL sel = PyObjCSelector_DefaultSelector(PyBytes_AsString(name_bytes));
 #else
-    SEL      sel = PyObjCSelector_DefaultSelector(PyObjC_Unicode_Fast_Bytes(name));
+    SEL sel = PyObjCSelector_DefaultSelector(PyObjC_Unicode_Fast_Bytes(name));
 #endif
 
     /* TODO: if sel.startswith('__') and sel.endswith('__'): look_in_runtime = False */
@@ -1700,6 +1730,8 @@ class_getattro(PyObject* self, PyObject* name)
             result = descr;
             Py_INCREF(result);
             goto done;
+        } else if (PyErr_Occurred()) {
+            return NULL;
         }
     }
 
@@ -1716,6 +1748,9 @@ class_getattro(PyObject* self, PyObject* name)
             ) {
             f = Py_TYPE(descr)->tp_descr_get;
         }
+        if (PyErr_Occurred()) {
+            return NULL;
+        }
     }
 
     if (descr == NULL) {
@@ -1731,6 +1766,9 @@ class_getattro(PyObject* self, PyObject* name)
             ) {
             f = Py_TYPE(descr)->tp_descr_get;
         }
+        if (PyErr_Occurred()) {
+            return NULL;
+        }
     }
 
     if (f != NULL) {
@@ -1840,7 +1878,7 @@ class_setattro(PyObject* self, PyObject* name, PyObject* value)
          */
         PyObject* newVal;
         Method curMethod;
-        Class  curClass;
+        Class curClass;
         int r;
         BOOL b;
 
@@ -1927,7 +1965,7 @@ class_richcompare(PyObject* self, PyObject* other, int op)
 {
     Class self_class;
     Class other_class;
-    int   v;
+    int v;
     PyObject* result;
 
     if (!PyObjCClass_Check(other)) {
@@ -2022,7 +2060,7 @@ class_compare(PyObject* self, PyObject* other)
 {
     Class self_class;
     Class other_class;
-    int   v;
+    int v;
 
     if (!PyObjCClass_Check(other)) {
         PyErr_SetString(PyExc_NotImplementedError, "Cmp with other");
@@ -2124,15 +2162,15 @@ cls_get_version(PyObject* self, void* closure __attribute__((__unused__)))
     }
 }
 
-static  int
+static int
 cls_set_version(PyObject* self, PyObject* newVal, void* closure __attribute__((__unused__)))
 {
     Class cls = PyObjCClass_GetClass(self);
-    int   val;
-    int   r;
+    int val;
+    int r;
 
     if (newVal == NULL) {
-        PyErr_SetString(PyExc_TypeError, "Cannot delete __version__  attribute");
+        PyErr_SetString(PyExc_TypeError, "Cannot delete __version__ attribute");
         return -1;
     }
 
@@ -2151,7 +2189,7 @@ cls_get_useKVO(PyObject* self, void* closure __attribute__((__unused__)))
     return PyBool_FromLong(((PyObjCClassObject*)self)->useKVO);
 }
 
-static  int
+static int
 cls_set_useKVO(PyObject* self, PyObject* newVal, void* closure __attribute__((__unused__)))
 {
     if (newVal == NULL) {
@@ -2314,7 +2352,7 @@ default_selector(PyObject* name)
 {
 #ifndef PyObjC_FAST_UNICODE_ASCII
     /* XXX: name_bytes */
-    SEL      sel = PyObjCSelector_DefaultSelector(PyBytes_AsString(name_bytes));
+    SEL sel = PyObjCSelector_DefaultSelector(PyBytes_AsString(name_bytes));
     return sel;
 #else
     return PyObjCSelector_DefaultSelector(PyObjC_Unicode_Fast_Bytes(name));
@@ -2608,7 +2646,7 @@ PyObjCClass_ListProperties(PyObject* aClass)
         return NULL;
     }
 
-    if (class_copyPropertyList == NULL) {
+    if (&class_copyPropertyList == NULL) {
         /* System without the 2.0 runtime and hence without
          * native properties
          */
@@ -2822,7 +2860,7 @@ PyObject*
 PyObjCClass_FindSelector(PyObject* cls, SEL selector, BOOL class_method)
 {
     PyObjCClassObject* info;
-    PyObject*          result;
+    PyObject* result;
 
     if (!PyObjCClass_Check(cls)) {
         PyErr_Format(PyObjCExc_InternalError,
@@ -2910,6 +2948,32 @@ PyObjCClass_FindSelector(PyObject* cls, SEL selector, BOOL class_method)
                 return value;
             }
         }
+        {
+            char selbuf[2048];
+            char* name;
+            PyObject* py_name;
+            name = PyObjC_SELToPythonName(selector, selbuf, sizeof(selbuf));
+            if (!name) continue;
+
+            py_name = PyText_FromString(name);
+            if (!py_name) {
+                PyErr_Clear();
+                continue;
+            }
+
+            if (class_method) {
+                value = PyObjCMetaClass_TryResolveSelector((PyObject*)Py_TYPE(c), py_name, selector);
+            } else {
+                value = PyObjCClass_TryResolveSelector(c, py_name, selector);
+            }
+            Py_DECREF(py_name);
+            if (value != NULL) {
+                Py_INCREF(value);
+                return value;
+            } else if (PyErr_Occurred()) {
+                return NULL;
+            }
+        }
     }
 
     /* If all else fails, ask the actual class (getattro also does this) */
@@ -3019,9 +3083,9 @@ update_convenience_methods(PyObject* cls)
         if (PyUnicode_Check(k)) {
             if (
                     PyObjC_is_ascii_string(k, "__dict__")
-                    ||    PyObjC_is_ascii_string(k, "__bases__")
-                    ||    PyObjC_is_ascii_string(k, "__slots__")
-                    ||    PyObjC_is_ascii_string(k, "__mro__")
+                    || PyObjC_is_ascii_string(k, "__bases__")
+                    || PyObjC_is_ascii_string(k, "__slots__")
+                    || PyObjC_is_ascii_string(k, "__mro__")
                  ) {
 
                 continue;
@@ -3031,7 +3095,7 @@ update_convenience_methods(PyObject* cls)
         } else if (PyString_Check(k)) {
             char* n = PyString_AS_STRING(k);
 
-            if (       strcmp(n, "__dict__") == 0
+            if (   strcmp(n, "__dict__") == 0
                 || strcmp(n, "__bases__") == 0
                 || strcmp(n, "__slots__") == 0
                 || strcmp(n, "__mro__") == 0
@@ -3081,7 +3145,7 @@ PyObjCClass_AddMethods(PyObject* classObject, PyObject** methods, Py_ssize_t met
     PyObject* extraDict = NULL;
     PyObject* metaDict = NULL;
 
-    targetClass  = PyObjCClass_GetClass(classObject);
+    targetClass = PyObjCClass_GetClass(classObject);
     if (targetClass == NULL) {
         return -1;
     }
@@ -3158,7 +3222,9 @@ PyObjCClass_AddMethods(PyObject* classObject, PyObject** methods, Py_ssize_t met
         objcMethod->type = strdup(
                 PyObjCSelector_Signature(aMethod));
 
-        PyObjC_RemoveInternalTypeCodes((char*)(objcMethod->type));
+        if (PyObjC_RemoveInternalTypeCodes((char*)(objcMethod->type)) == -1)  {
+            goto cleanup_and_return_error;
+        }
         if (objcMethod->type == NULL) {
             goto cleanup_and_return_error;
         }
