@@ -5,7 +5,7 @@
  * Central include file for PyObjC.
  */
 
-#define OBJC_VERSION "5.1.1"
+#define OBJC_VERSION "5.2"
 
 #define PY_SSIZE_T_CLEAN
 #include <Python.h>
@@ -68,6 +68,7 @@
 #include "proxy-registry.h"
 #include "objc_support.h"
 #include "pointer-support.h"
+#include "released-buffer.h"
 #include "OC_PythonObject.h"
 #include "OC_PythonArray.h"
 #include "OC_BuiltinPythonArray.h"
